#!/usr/bin/env python
# -*- coding: utf-8 -*-

#  PostmasterGS Stamp Album Page Generator for Scribus v1.04 (20130706)
#  Brought to you by GermanStamps.net -- http://www.germanstamps.net
#
#  This script is free to use and modify.  However I have invested significant
#  time and effort ino this project, and I simply ask that you leave this block
#  of text intact as recognition of the source of the code.

from ij import IJ, ImagePlus
from ij.io import Opener
from ij.io import OpenDialog
from ij.io import FileSaver
from ij.gui import ImageWindow
from ij.plugin.filter import Binary
from math import *
import os

def Initialize():
	# Set your PMGS directory here.  Be sure to include double slashes, including at the end of the path 
	# On a PC, it should be in a format like this -- "C:\\Users\\Jed\\Documents\\Stamps\\Temp\\"
	# On a Mac, it should be in a format like this -- "/Users/Jed/Documents/Stamps/Temp/"
	global strPMGSDir
	strPMGSDir = "C:\\Users\\Jed\Desktop\\PMGSScribus\\"
	
	global tempDir
	tempDir = strPMGSDir + "PMGSTemp\\"

	# Set the resolution of your photos here in ppi/dpi.  The next version will autodetect this, but there are some limitations in the current imaging software.
	global res
	res = float(600)
	
	global imp
	global imgWidth
	global imgHeight

	od = OpenDialog("Choose a file.", None)
	global filename
	filename = od.getFileName()
	if filename is None:
		return
	else:
		strExtension = filename
		strExtensionI = filename.find(".")+1
		strExtensionL = len(filename)
		global strExtension
		strExtension = strExtension[strExtensionI:strExtensionL]
		global directory
		directory = od.getDirectory()
		global filepath
		filepath = directory + filename
	IJ.open(filepath);
	imp = IJ.getImage()	
	
	global imgWidth
	imgWidth = imp.getWidth()
	global imgHeight
	imgHeight = imp.getHeight()

	global stpIB
	stpIB = 0
	global IBWd
	IBWd = 0
	global ang
	ang = 0

	SetupImage()

	FindLeft(255,0,0)
	FindRight(255,0,0)
	FindTop(255,0,0)
	FindBottom(255,0,0)

	while stpIB < 1:
		IJ.run("Select None");
		if FTx > FBx:
			IJ.run(imp, "Rotate...", "angle=0.5 grid=1 interpolate enlarge");
			global ang
			ang = ang + 0.5
		else:
			IJ.run(imp, "Rotate...", "angle=-0.5 grid=1 interpolate enlarge");
			global ang
			ang = ang - 0.5
		CenterBox()
	ang = str(ang)

	FindLeft(255,0,0)
	FindRight(255,0,0)
	FindTop(255,0,0)
	FindBottom(255,0,0)

	stmHeight = FBy - FTy
	stmWidth = FRx - FLx
	stmHeight = round((stmHeight/res * 25.4),2)
	stmWidth = round((stmWidth/res * 25.4),2)

	IJ.run("Close");

	IJ.open(filepath);
	imp1 = IJ.getImage()
	IJ.run("Make Binary");
	IJ.run("Fill Holes");
	IJ.run("Erode");
	IJ.run("Erode");
	IJ.run("Erode");
	IJ.run("Dilate");
	IJ.run("Despeckle");
	IJ.run("Despeckle");
	IJ.run("Despeckle");
	IJ.run("Despeckle");
	IJ.run("Despeckle");
	IJ.run(imp1, "Rotate...", "angle=" + ang + " grid=1 interpolate enlarge");
	if os.path.isdir(tempDir) is False:
		os.makedirs(tempDir)
	IJ.saveAs("PNG", tempDir + "binary_temp.png");

	IJ.open(filepath);
	imp2 = IJ.getImage()
	IJ.run(imp2, "Rotate...", "angle=" + ang + " grid=1 interpolate enlarge");
	IJ.saveAs("JPG", tempDir + filename);

	ic = ImageCalculator()
	imp3 = ic.run("Subtract", imp2, imp1);
	IJ.run("Select Bounding Box (guess background color)")
	IJ.run("Crop")
	imp3 = IJ.getImage()

	'''
	if strExtension == "jpg":
		fs = FileSaver(imp3)
		fs.saveAsJpeg()
	elif strExtenstion == "jpeg":
		fs = FileSaver(imp3)
		fs.saveAsJpeg()
	elif strExtenstion == "gif":
		fs = FileSaver(imp3)
		fs.saveAsGif()
	elif strExtenstion == "Bmp":
		fs = FileSaver(imp3)
		fs.saveAsBmp()
	elif strExtenstion == "png":
		fs = FileSaver(imp3)
		fs.saveAsPng()
	elif strExtenstion == "tiff":
		fs = FileSaver(imp3)
		fs.saveAsTiff()
	'''

	sd = SaveDialog("Save file.", directory, filename, None)
	filenameS = sd.getFileName()
	directoryS = sd.getDirectory()
	if filenameS is None:
		return
	else:
		filepathS = directoryS + filenameS
		if strExtension == "png":
			IJ.saveAs("PNG", filepathS);
		else:
			IJ.saveAs("Jpeg", filepathS);		
		writeString = str(filepathS) + "|" + str(stmHeight) + "|" + str(stmWidth)
		f1 = open(strPMGSDir + "PMGSSAPG_stamp_dimensions.txt", 'a')
		f1.write(writeString + '\n')
		f1.close()
	IJ.run("Close All");
	os.remove(tempDir + "binary_temp.png")
	os.remove(tempDir + filename)
	os.remove(tempDir)	
    

def SetupImage():
	IJ.run("Make Binary");
	IJ.run("Fill Holes");
	IJ.run("Erode");
	IJ.run("Erode");
	IJ.run("Erode");
	IJ.run("Dilate");
	IJ.run("Despeckle");
	IJ.run("Despeckle");
	IJ.run("Despeckle");
	IJ.run("Despeckle");
	IJ.run("Despeckle");

	IJ.run("Select Bounding Box (guess background color)");
	IJ.run("Crop")

def CenterBox():
	ip = imp.getProcessor().convertToFloat()
	IJ.run("Select Bounding Box (guess background color)");
	IJ.run("Crop")
	CMx = str(ip.getStatistics().roiWidth / 2)
	CMy = str(ip.getStatistics().roiHeight / 2)
	IJ.run("Select None");
	IJ.run("Specify...", "width=1 height=1 x=" + CMx + " y=" + CMy + " centered");
	IBMin = 1
	while IBMin > 0:
		IJ.run("Enlarge...", "enlarge=1");
		IBStats = imp.getStatistics()
		IBMin = IBStats.min

	IBW = IBStats.roiWidth
	global IBWd
	if IBWd <= IBW:
		IBWd = IBW
		stpIB = 0
	else:
		IJ.run("Select None");
		if FTx > FBx:
			IJ.run(imp, "Rotate...", "angle=-1 grid=1 interpolate enlarge");
			global ang
			ang = ang - 1
		else:
			IJ.run(imp, "Rotate...", "angle=1 grid=1 interpolate enlarge");
			global ang
			ang = ang + 1
		global stpIB
		stpIB = 1
		IJ.run("Sharpen");
		IJ.run("Sharpen");

def FindLeft(r,g,b):
	for FLx in range(1, imgWidth):
		for FLy in range (1, imgHeight):
			global FLpx
			FLpx = imp.getPixel(FLx,FLy)
			if FLpx[0] == r and FLpx[1] == g and FLpx[2] == b:
				global FLx
				global FLy
				return (FLx,FLy)

def FindRight(r,g,b):
	for FRx in range(imgWidth, 1, -1):
		for FRy in range (imgHeight, 1, -1):
			global FRpx
			FRpx = imp.getPixel(FRx,FRy)
			if FRpx[0] == r and FRpx[1] == g and FRpx[2] == b:
				global FRx
				global FRy
				return (FRx,FRy)
				
def FindTop(r,g,b):
	for FTy in range (1, imgHeight):
		for FTx in range(1, imgWidth):
			global FTpx
			FTpx = imp.getPixel(FTx,FTy)
			if FTpx[0] == r and FTpx[1] == g and FTpx[2] == b:
				global FTx
				global FTy
				return (FTx,FTy)

def FindBottom(r,g,b):
	for FBy in range (imgHeight, 1, -1):
		for FBx in range(imgWidth, 1, -1):
			global FBpx
			FBpx = imp.getPixel(FBx,FBy)
			if FBpx[0] == r and FBpx[1] == g and FBpx[2] == b:
				global FBx
				global FBy
				return (FBx,FBy)

Initialize()